import { createRoot } from "react-dom/client";
import App from "./App";
import "./index.css";
import { QueryClientProvider } from "@tanstack/react-query";
import { queryClient } from "./lib/queryClient";
import { reactQueryConfig } from "./production";
import React from 'react';
import { 
  initRuntimeOptimizations, 
  optimizeLCPElement, 
  removeDevOnlyCode, 
  optimizeAllImages,
  cleanupUnusedListeners,
  deferNonCriticalCSS
} from "./runtime-optimizations";
// Import production build optimizers
import { stripDevOnlyCode } from "./production.buildOptimizer";
import { 
  enforceProductionMode, 
  removeUnusedJavaScript, 
  isProductionMode 
} from "./lib/productionEnvironment";

// Apply production configurations to query client
if (process.env.NODE_ENV === 'production') {
  Object.assign(queryClient.getDefaultOptions(), reactQueryConfig.defaultOptions);
}

// Enable React profiling only in development
const StrictModeWrapper: React.FC<{children: React.ReactNode}> = process.env.NODE_ENV === 'development' 
  ? ({ children }) => <React.StrictMode>{children}</React.StrictMode>
  : ({ children }) => <>{children}</>;

// Performance optimizations that run on startup
if (typeof window !== 'undefined') {
  // Initialize core performance optimizations as early as possible
  removeDevOnlyCode();
  
  // Apply additional production-specific optimizations
  if (process.env.NODE_ENV === 'production' || isProductionMode()) {
    // Ensure React is running in production mode
    enforceProductionMode();
    
    // Apply optimizations from both modules
    stripDevOnlyCode();
    
    // Remove unused JavaScript
    setTimeout(removeUnusedJavaScript, 0);
  }
  
  // Optimize CSS loading
  if (process.env.NODE_ENV === 'production') {
    // Preload critical CSS
    const cssLink = document.createElement('link');
    cssLink.rel = 'preload';
    cssLink.as = 'style';
    cssLink.href = '/src/index.css';
    document.head.appendChild(cssLink);
    
    // Directly add critical CSS as a stylesheet
    const criticalCssLink = document.createElement('link');
    criticalCssLink.rel = 'stylesheet';
    criticalCssLink.href = '/critical.css';
    document.head.appendChild(criticalCssLink);

    // Use priority hints for important resources
    const preloadLink = document.createElement('link');
    preloadLink.rel = 'preload';
    preloadLink.as = 'style';
    preloadLink.href = '/critical.css';
    document.head.appendChild(preloadLink);
    
    // Additional optimization: use the deferNonCriticalCSS function
    deferNonCriticalCSS('/assets/non-critical.css');
  }
  
  // Initialize runtime optimizations once DOM content is loaded
  window.addEventListener('DOMContentLoaded', () => {
    // Find and optimize the LCP element (Largest Contentful Paint)
    const lcpHeading = document.querySelector('h1.text-3xl.md\\:text-5xl');
    if (lcpHeading) {
      optimizeLCPElement(lcpHeading as HTMLElement);
    }
    
    // Also optimize any hero images
    const heroImgs = document.querySelectorAll('.hero img, header img');
    heroImgs.forEach(img => {
      optimizeLCPElement(img as HTMLImageElement);
    });
    
    // Optimize image loading
    optimizeAllImages();
  });
  
  // Run more comprehensive optimizations after the page is fully loaded
  window.addEventListener('load', () => {
    // Initialize all optimizations in one call
    initRuntimeOptimizations();
    
    // Run additional production-specific optimizations on load
    if (process.env.NODE_ENV === 'production' || isProductionMode()) {
      // Remove unused JavaScript files more aggressively
      removeUnusedJavaScript();
    }
    
    // Delay non-critical operations to improve initial page performance
    setTimeout(() => {
      // Clean up event listeners on non-interactive elements
      cleanupUnusedListeners();
      
      // In production, do a final cleanup of development scripts
      if (process.env.NODE_ENV === 'production' || isProductionMode()) {
        removeUnusedJavaScript();
      }
    }, 2000);
  });
}

createRoot(document.getElementById("root")!).render(
  <StrictModeWrapper>
    <QueryClientProvider client={queryClient}>
      <App />
    </QueryClientProvider>
  </StrictModeWrapper>
);
